"use strict";
// Copyright 2020 Google Inc. Use of this source code is governed by an
// MIT-style license that can be found in the LICENSE file or at
// https://opensource.org/licenses/MIT.
Object.defineProperty(exports, "__esModule", { value: true });
const rxjs_1 = require("rxjs");
const varint = require("varint");
const utils_1 = require("../../test/utils");
const message_transformer_1 = require("./message-transformer");
const proto = require("./vendor/embedded_sass_pb");
describe('message transformer', () => {
    let messages;
    function validInboundMessage(source) {
        return new proto.InboundMessage({
            message: {
                case: 'compileRequest',
                value: new proto.InboundMessage_CompileRequest({
                    input: {
                        case: 'string',
                        value: new proto.InboundMessage_CompileRequest_StringInput({
                            source,
                        }),
                    },
                }),
            },
        });
    }
    describe('encode', () => {
        let encodedProtobufs;
        beforeEach(() => {
            encodedProtobufs = [];
            messages = new message_transformer_1.MessageTransformer(new rxjs_1.Observable(), buffer => encodedProtobufs.push(buffer));
        });
        it('encodes an InboundMessage to buffer', () => {
            const message = validInboundMessage('a {b: c}');
            messages.writeInboundMessage([1234, message]);
            expect(encodedProtobufs).toEqual([
                Uint8Array.from([...varint.encode(1234), ...message.toBinary()]),
            ]);
        });
    });
    describe('decode', () => {
        let protobufs$;
        let decodedMessages;
        beforeEach(() => {
            protobufs$ = new rxjs_1.Subject();
            messages = new message_transformer_1.MessageTransformer(protobufs$, () => { });
            decodedMessages = [];
        });
        it('decodes buffer to OutboundMessage', done => {
            messages.outboundMessages$.subscribe({
                next: message => decodedMessages.push(message),
                complete: () => {
                    expect(decodedMessages.length).toBe(1);
                    const [id, message] = decodedMessages[0];
                    expect(id).toBe(1234);
                    expect(message.message.case).toBe('compileResponse');
                    const response = message.message
                        .value;
                    expect(response.result.case).toBe('success');
                    expect(response.result
                        .value.css).toBe('a {b: c}');
                    done();
                },
            });
            protobufs$.next(Uint8Array.from([
                ...varint.encode(1234),
                ...validInboundMessage('a {b: c}').toBinary(),
            ]));
            protobufs$.complete();
        });
        describe('protocol error', () => {
            it('fails on invalid buffer', done => {
                (0, utils_1.expectObservableToError)(messages.outboundMessages$, 'Compiler caused error: Invalid compilation ID varint: RangeError: ' +
                    'Could not decode varint.', done);
                protobufs$.next(Buffer.from([-1]));
            });
        });
    });
});
//# sourceMappingURL=message-transformer.test.js.map